package main

import (
	"flag"
	"fmt"
	"net"
	"os"

	"github.com/labstack/echo/v4"
	"github.com/likexian/whois-go"
	whoisparser "github.com/likexian/whois-parser-go"
)

// GetIPInfo returns IP address informations
func GetIPInfo(c echo.Context) (ip IP, err error) {
	if c.QueryParam("ip") != "" {
		ip.IP = c.QueryParam("ip")
	} else {
		ip.IP = c.RealIP()
	}

	err = ip.CheckIPAddress()
	if err != nil {
		return IP{}, err
	}

	err = ip.GetHostname()
	if err != nil {
		return IP{}, err
	}

	return
}

// DipAsIndex ...
func DipAsIndex(c echo.Context) (err error) {
	var ip IP
	ip, err = GetIPInfo(c)
	if err != nil {
		return c.HTML(500, fmt.Sprintf(
			`<html>
<body>
	<h1>Error occured: %s</h1>
</body>
</html>
`, err))
	}
	return c.HTML(200, fmt.Sprintf(
		`<html>
<body>
	<h1>IP: %s</h1>
	<h2>Reverse DNS: %s</h2>
</body>
</html>
`, ip.IP, ip.Hostname))
}

// DipAsJSON ...
func DipAsJSON(c echo.Context) (err error) {
	var ip IP
	ip, err = GetIPInfo(c)
	if err != nil {
		return c.JSON(500, fmt.Sprintf("Error occured: %s", err))
	}
	return c.JSON(200, ip)
}

// CheckIPAddress ...
func (ip *IP) CheckIPAddress() (err error) {
	addr := net.ParseIP(ip.IP)
	if addr == nil {
		return fmt.Errorf("Supplied string isn't an IP address")
	}
	return
}

// GetHostname assign ip.Hostname value
func (ip *IP) GetHostname() (err error) {
	revip, err := net.LookupAddr(ip.IP)
	if err != nil {
		return fmt.Errorf(fmt.Sprintf("Supplied address %s doesn't have a valid reverse DNS entry", ip.IP))
	}
	ip.Hostname = revip[0]
	return
}

// GetWhois not implemened yet
func (ip *IP) GetWhois() (err error) {
	resultraw, err := whois.Whois("exemple.com")
	//resultraw, err := whois.Whois(ip.IP)
	fmt.Println(resultraw)
	result, err := whoisparser.Parse(resultraw)
	if err != nil {
		return
	}
	fmt.Println(result)
	return
}

// Usage displays possible arguments
func Usage() {
	flag.PrintDefaults()
	os.Exit(1)
}

// IP ...
type IP struct {
	IP       string `json:"ip"`
	Hostname string `json:"hostname"`
}
